/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.hadoop.yarn.server.timelineservice.storage.flow;

import org.apache.hadoop.yarn.server.timelineservice.storage.common.RowKeyPrefix;

/**
 * A prefix partial rowkey for flow activities.
 */
public class FlowActivityRowKeyPrefix extends FlowActivityRowKey implements
    RowKeyPrefix<FlowActivityRowKey> {

  /**
   * Constructs a row key prefix for the flow activity table as follows:
   * {@code clusterId!dayTimestamp!}.
   *
   * @param clusterId Cluster Id.
   * @param dayTs Start of the day timestamp.
   */
  public FlowActivityRowKeyPrefix(String clusterId, Long dayTs) {
    super(clusterId, dayTs, null, null, false);
  }

  /**
   * Constructs a row key prefix for the flow activity table as follows:
   * {@code clusterId!}.
   *
   * @param clusterId identifying the cluster
   */
  public FlowActivityRowKeyPrefix(String clusterId) {
    super(clusterId, null, null, null, false);
  }

  /*
   * (non-Javadoc)
   *
   * @see
   * org.apache.hadoop.yarn.server.timelineservice.storage.application.
   * RowKeyPrefix#getRowKeyPrefix()
   */
  public byte[] getRowKeyPrefix() {
    return super.getRowKey();
  }

}
